/*
 * Post-Load command for Fd.   Checks and installs /dev/fd/#
 *
 *	Author: Mark Salyzyn  mark@bohica.net  December 5 1995
 */

#import <driverkit/IODeviceMaster.h>
#import <driverkit/IODevice.h>
#import <driverkit/IOConfigTable.h>
#import <errno.h>
#import <libc.h>
#import <stdio.h>
#import "../FdParm.h"
#ifdef Use_syslog
# import <syslog.h>
#else
# undef LOG_CRIT
# define LOG_CRIT	stderr
# undef LOG_WARNING
# define LOG_WARNING	stderr
# undef LOG_DEBUG
# define LOG_DEBUG	stderr
# undef syslog
# define syslog		fprintf
#endif
#include	<sys/types.h>
#include	<sys/stat.h>

#define DEBUG_LOG	0

int main(int argc, char **argv)
{	id 		deviceMaster = [IODeviceMaster new] ;
	IOObjectNumber 	objectNum ;
	IOString 	kind ;
	IOReturn 	rtn ;
	char		name[sizeof(FdDeviceName)+1] ;
	unsigned 	array[1], nofile, minorNum, majorNum ;
	unsigned	count = 1 ;
	const char *	instance ;
	char		nodeName[16] ;
	extern char *	strcpy() ;

	/*
	 *	Parse out the Instance information from the passed arguments
	 */
	instance = "0" ;
	if ((argc > 1) && (strncmp (argv[1], "Instance=", 9) == 0))
		instance = argv[1] + 9 ;
	(void)strcpy (name, FdDeviceName) ;
	(void)strcpy (kind, FdDeviceKind) ;
	name[sizeof(name)-1] = '\0' ;
	name[sizeof(name)-2] = *instance ;
	if (rtn = [deviceMaster lookUpByDeviceName : name
				      objectNumber : &objectNum
				        deviceKind : &kind]) {
		syslog(LOG_CRIT,"Error contacting %s (%s)\n",
			name, [IODevice stringFromReturn : rtn]) ;
		exit(1) ;	
	}
	if ((rtn = [deviceMaster getIntValues : array
			         forParameter : "IONumberDevice"
			         objectNumber : objectNum
			                count : &count])	// in/out
	 || (count != 1)) {
		syslog(LOG_WARNING,"Error performing NOFILE (%s), Using %d\n",
			[IODevice stringFromReturn : rtn], NOFILE) ;
		nofile = NOFILE ;
	} else	nofile = array[0] ;
	if ((rtn = [deviceMaster getIntValues : array
			         forParameter : "IOMajorDevice"
			         objectNumber : objectNum
			                count : &count])	// in/out
	 || (count != 1)) {
		syslog(LOG_CRIT,"Error performing IOMajorDevice (%s)\n",
			[IODevice stringFromReturn : rtn]) ;
		exit(1) ;	
	}
	majorNum = array[0] ;
	unlink ("/dev/stdin") ;
	unlink ("/dev/stdout") ;
	unlink ("/dev/stderr") ;
	for(minorNum=0 ; minorNum<nofile ; minorNum++) {
		sprintf(nodeName, "/dev/fd/%d", minorNum) ;
		unlink(nodeName) ;
	}
	/*
	 *	There is a bug in NS3.3 that panics the Kernel if we ask
	 *	for mknod (?, 0666|S_IFCHR, 0x0110). Limit us to 16 file
	 *	descriptors in this case and call it a day.
	 */
	nofile = 16 ;	/* What a bloody waste */
	rmdir ("/dev/fd") ;
	unlink ("/dev/fd") ;	/* In case it is a file */
	if (mkdir ("/dev/fd", S_IFDIR|0755) != 0)
		syslog(LOG_WARNING,"Can\'t create /dev/fd directory\r\n") ;
	for(minorNum=0 ; minorNum<nofile ; minorNum++) {
		sprintf(nodeName, "/dev/fd/%d", minorNum) ;
		if(mknod(nodeName, 0666|S_IFCHR, makedev (majorNum, minorNum)))
			syslog(LOG_WARNING,"Can\'t create %s\r\n", nodeName) ;
		if(chmod(nodeName,0666))
			syslog(LOG_WARNING,"Can\'t ensure %s is +rw\r\n", nodeName);
	}
	link ("/dev/fd/0", "/dev/stdin") ;
	link ("/dev/fd/1", "/dev/stdout") ;
	link ("/dev/fd/2", "/dev/stderr") ;
	return 0 ;
}
